# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for BudouX."""

from enum import Enum

SEP = '▁'
"""The separator string to specify breakpoints."""

INVALID = '▔'
"""The invalid feature string."""


class Result(Enum):
  """An enum to represent the type of inference result."""
  UNKNOWN = 'U'
  POSITIVE = 'B'
  NEGATIVE = 'O'
